@@ -11,7 +11,7 @@
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      Affero General Public License for more details.
-    
+
      You should have received a copy of the GNU Affero General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -43,9 +43,8 @@
 #define GNUNET_IDENTITY_SERVICE_H
 
 #ifdef __cplusplus
-extern "C"
-{
-#if 0                           /* keep Emacsens' auto-indent happy */
+extern "C" {
+#if 0 /* keep Emacsens' auto-indent happy */
 }
 #endif
 #endif
@@ -56,7 +55,7 @@ extern "C"
 /**
  * Version number of GNUnet Identity API.
  */
-#define GNUNET_IDENTITY_VERSION 0x00000000
+#define GNUNET_IDENTITY_VERSION 0x00000100
 
 /**
  * Handle to access the identity service.
@@ -101,7 +100,7 @@ GNUNET_IDENTITY_ego_get_anonymous (void);
  */
 void
 GNUNET_IDENTITY_ego_get_public_key (const struct GNUNET_IDENTITY_Ego *ego,
-				    struct GNUNET_CRYPTO_EcdsaPublicKey *pk);
+                                    struct GNUNET_CRYPTO_EcdsaPublicKey *pk);
 
 
 /**
@@ -138,11 +137,10 @@ GNUNET_IDENTITY_ego_get_public_key (const struct GNUNET_IDENTITY_Ego *ego,
  *                   NULL if the user just deleted the ego and it
  *                   must thus no longer be used
  */
-typedef void
-(*GNUNET_IDENTITY_Callback)(void *cls,
-                            struct GNUNET_IDENTITY_Ego *ego,
-                            void **ctx,
-                            const char *name);
+typedef void (*GNUNET_IDENTITY_Callback) (void *cls,
+                                          struct GNUNET_IDENTITY_Ego *ego,
+                                          void **ctx,
+                                          const char *name);
 
 
 /**
@@ -155,8 +153,8 @@ typedef void
  */
 struct GNUNET_IDENTITY_Handle *
 GNUNET_IDENTITY_connect (const struct GNUNET_CONFIGURATION_Handle *cfg,
-			 GNUNET_IDENTITY_Callback cb,
-			 void *cb_cls);
+                         GNUNET_IDENTITY_Callback cb,
+                         void *cb_cls);
 
 
 /**
@@ -170,9 +168,9 @@ GNUNET_IDENTITY_connect (const struct GNUNET_CONFIGURATION_Handle *cfg,
  */
 struct GNUNET_IDENTITY_Operation *
 GNUNET_IDENTITY_get (struct GNUNET_IDENTITY_Handle *id,
-		     const char *service_name,
-		     GNUNET_IDENTITY_Callback cb,
-		     void *cb_cls);
+                     const char *service_name,
+                     GNUNET_IDENTITY_Callback cb,
+                     void *cb_cls);
 
 
 /**
@@ -182,9 +180,7 @@ GNUNET_IDENTITY_get (struct GNUNET_IDENTITY_Handle *id,
  * @param cls closure
  * @param emsg NULL on success, otherwise an error message
  */
-typedef void
-(*GNUNET_IDENTITY_Continuation)(void *cls,
-                                const char *emsg);
+typedef void (*GNUNET_IDENTITY_Continuation) (void *cls, const char *emsg);
 
 
 /**
@@ -199,10 +195,10 @@ typedef void
  */
 struct GNUNET_IDENTITY_Operation *
 GNUNET_IDENTITY_set (struct GNUNET_IDENTITY_Handle *id,
-		     const char *service_name,
-		     struct GNUNET_IDENTITY_Ego *ego,
-		     GNUNET_IDENTITY_Continuation cont,
-		     void *cont_cls);
+                     const char *service_name,
+                     struct GNUNET_IDENTITY_Ego *ego,
+                     GNUNET_IDENTITY_Continuation cont,
+                     void *cont_cls);
 
 
 /**
@@ -214,6 +210,20 @@ void
 GNUNET_IDENTITY_disconnect (struct GNUNET_IDENTITY_Handle *h);
 
 
+/**
+ * Function called once the requested operation has
+ * been completed.
+ *
+ * @param cls closure
+ * @param pk private key, NULL on error
+ * @param emsg error message, NULL on success
+ */
+typedef void (*GNUNET_IDENTITY_CreateContinuation) (
+  void *cls,
+  const struct GNUNET_CRYPTO_EcdsaPrivateKey *pk,
+  const char *emsg);
+
+
 /**
  * Create a new ego with the given name.
  *
@@ -225,9 +235,9 @@ GNUNET_IDENTITY_disconnect (struct GNUNET_IDENTITY_Handle *h);
  */
 struct GNUNET_IDENTITY_Operation *
 GNUNET_IDENTITY_create (struct GNUNET_IDENTITY_Handle *id,
-			const char *name,
-			GNUNET_IDENTITY_Continuation cont,
-			void *cont_cls);
+                        const char *name,
+                        GNUNET_IDENTITY_CreateContinuation cont,
+                        void *cont_cls);
 
 
 /**
@@ -242,10 +252,10 @@ GNUNET_IDENTITY_create (struct GNUNET_IDENTITY_Handle *id,
  */
 struct GNUNET_IDENTITY_Operation *
 GNUNET_IDENTITY_rename (struct GNUNET_IDENTITY_Handle *id,
-			const char *old_name,
-			const char *new_name,
-			GNUNET_IDENTITY_Continuation cb,
-			void *cb_cls);
+                        const char *old_name,
+                        const char *new_name,
+                        GNUNET_IDENTITY_Continuation cb,
+                        void *cb_cls);
 
 
 /**
@@ -259,9 +269,9 @@ GNUNET_IDENTITY_rename (struct GNUNET_IDENTITY_Handle *id,
  */
 struct GNUNET_IDENTITY_Operation *
 GNUNET_IDENTITY_delete (struct GNUNET_IDENTITY_Handle *id,
-			const char *name,
-			GNUNET_IDENTITY_Continuation cb,
-			void *cb_cls);
+                        const char *name,
+                        GNUNET_IDENTITY_Continuation cb,
+                        void *cb_cls);
 
 
 /**
@@ -284,9 +294,9 @@ GNUNET_IDENTITY_cancel (struct GNUNET_IDENTITY_Operation *op);
  * @param cls closure
  * @param ego NULL on error / ego not found
  */
-typedef void
-(*GNUNET_IDENTITY_EgoCallback)(void *cls,
-                               const struct GNUNET_IDENTITY_Ego *ego);
+typedef void (*GNUNET_IDENTITY_EgoCallback) (
+  void *cls,
+  const struct GNUNET_IDENTITY_Ego *ego);
 
 /**
  * Handle for ego lookup.
@@ -305,9 +315,9 @@ struct GNUNET_IDENTITY_EgoLookup;
  */
 struct GNUNET_IDENTITY_EgoLookup *
 GNUNET_IDENTITY_ego_lookup (const struct GNUNET_CONFIGURATION_Handle *cfg,
-			    const char *name,
-			    GNUNET_IDENTITY_EgoCallback cb,
-			    void *cb_cls);
+                            const char *name,
+                            GNUNET_IDENTITY_EgoCallback cb,
+                            void *cb_cls);
 
 
 /**
@@ -318,8 +328,53 @@ GNUNET_IDENTITY_ego_lookup (const struct GNUNET_CONFIGURATION_Handle *cfg,
 void
 GNUNET_IDENTITY_ego_lookup_cancel (struct GNUNET_IDENTITY_EgoLookup *el);
 
+/**
+ * Function called with the result.
+ *
+ * @param cls closure
+ * @param ego NULL on error / ego not found
+ * @param ego_name NULL on error, name of the ego otherwise
+ */
+typedef void (*GNUNET_IDENTITY_EgoSuffixCallback) (
+  void *cls,
+  const struct GNUNET_CRYPTO_EcdsaPrivateKey *priv,
+  const char *ego_name);
 
-#if 0                           /* keep Emacsens' auto-indent happy */
+
+/**
+ * Handle for suffix lookup.
+ */
+struct GNUNET_IDENTITY_EgoSuffixLookup;
+
+
+/**
+ * Obtain the ego with the maximum suffix match between the
+ * ego's name and the given domain name @a suffix.  I.e., given
+ * a @a suffix "a.b.c" and egos with names "d.a.b.c", "b.c" and "c",
+ * we return the ego for "b.c".
+ *
+ * @param cfg configuration to use
+ * @param suffix for which domain name suffix is an identity wanted
+ * @param cb function to call with the result (will only be called once)
+ * @param cb_cls closure for @a cb
+ * @return handle to abort the operation
+ */
+struct GNUNET_IDENTITY_EgoSuffixLookup *
+GNUNET_IDENTITY_ego_lookup_by_suffix (const struct GNUNET_CONFIGURATION_Handle *cfg,
+				      const char *suffix,
+				      GNUNET_IDENTITY_EgoSuffixCallback cb,
+				      void *cb_cls);
+
+
+/**
+ * Abort ego suffix lookup attempt.
+ *
+ * @param el handle for lookup to abort
+ */
+void
+GNUNET_IDENTITY_ego_lookup_by_suffix_cancel (struct GNUNET_IDENTITY_EgoSuffixLookup *el);
+
+#if 0 /* keep Emacsens' auto-indent happy */
 {
 #endif
 #ifdef __cplusplus
